/*
* Titel........ Small radio base station based on Hope RF's RFM01 receiver
* Version...... 01
* Dateiname.... main.c
* erstellt am.. 2014-02-21
* gendert am.. 2017-02-20
* Platform..... ATMEL ATtiny261A
* Autor........ Clemens B Fahr
* email........ clemens.fahr[at]fahrnet.de
* Kommentar.... created based on RFM_wireless_small_base_01 SVN 465;
* .............	add bug fix (potentially for bug_00006_20140305) for illegally 
* .............	long or short telegrams with added RFM init;
* .............	modify radio parameters to match with sensors (data rate, sensitivity, bandwidth)
*/

// includes
//------------------------------------------------------------------
#include "global.h"
#include "main.h"
#include "lowlevel.h"

// interrupt service routines -- ISRs
//------------------------------------------------------------------
// mind spelling SIGNAL command in capitals only
// all interrupt sources, though not used, should be captured

ISR(TIMER1_OVF_vect)
{
        LOWLEVEL_toggle_LED1(TRUE);
}

ISR(INT0_vect)
{
        uint8_t i, aByte;
        uint16_t RFMstatus;
        
        cli();
        
        /* total received telegram consist of
	*
	* byte  | 0 |  1-2 | 3-4 |   5:n   | n+1 |
	* reps  | L |  ID  | UID | payload | CRC |
	*
	* - L is 1byte payload length starting byte 0 up to incl. byte n+1
	* - ID is 2byte common identifier for the TX-node
	* - UID is 2byte unique identifiers for the TX-node
	* - CRC is the modular sum of bytes 0 to incl. n
	*/
        
        /* receiving one byte at a time here
        * the status flag (16 bits) is checked prior
        */
        
        // the nIRQ is low now, hence read the status and FIFO
        RFMstatus = 0;
        RFM_SDI(LOW);
        RFM_SCK(LOW);
        RFM_SEL(LOW);
        for (i=0;i<16;i++) {
                RFMstatus <<= 1;
                if (RFM_SDO_HIGH) {
                        RFMstatus |= 0x0001;
                }
                RFM_SCK(HIGH);
                RFM_SCK(LOW);
        }
        // if the FFIT is set in status do read 8bits from fifo
        if (RFMstatus & 0x8000) {
                aByte = 0;
                //nop();
                for (i=0;i<8;i++) {
                        aByte <<= 1;
                        if ( RFM_SDO_HIGH ) {
                                aByte |= 0x01;
                        }
                        RFM_SCK(HIGH);
                        RFM_SCK(LOW);
                }
                RX_buffer[numberReceivedBytes] = aByte;
        }
        RFM_SEL(HIGH);
        numberReceivedBytes++;
        if ( numberReceivedBytes == RX_buffer[0] ) { 
                // according to telegram definition index 0 contains the telegram length
                telegramComplete = TRUE;
        }
        if ( (RX_buffer[0] <= 6) || (RX_buffer[0] > MAX_BUFFER_SIZE) ) {
                // invalid telegram or invalid telegram size
                LOWLEVEL_init_RFM();
        }
        
        sei();
}

//------------------------------------------------------------------

int main(void)
{
	uint8_t i, checksum=0;
	
	LOWLEVEL_init_all();  // run all low level initialisations
        
        sei();

	while  (1) {
		
		// if number of bytes indicated is equivalent to numberReceivedBytes continue, 
                // i.e. telegramComplete is true
		if ( telegramComplete ) {
                
                        cli();
                
                        LOWLEVEL_RFM_stop_RX();
			
			// analyse the checksum of the received telegram
			checksum = 0;
			for (i=0;i<(numberReceivedBytes-1);i++) {
				checksum += RX_buffer[i];
				// LOWLEVEL_UART_TX(RX_buffer[i]);
			}
			
			// if checksum of received telegram matches checksum (recalculated) continue
			if ( checksum == RX_buffer[numberReceivedBytes-1] ) {
			
				// send the "start of telegram" mark
				LOWLEVEL_UART_TX(SOT);	
			
				// now forward through serial UART without alterations
				for (i=0;i<numberReceivedBytes;i++) {
					LOWLEVEL_UART_TX(RX_buffer[i]);
				}
			}
                        
            LOWLEVEL_init_RFM();	// re-init the RFM01 module and related variables
                        
            LOWLEVEL_toggle_LED1(FALSE);
                        
            sei();
		}
	}
	
	return 0;  //never reached
}
